#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, re, requests, time, hashlib, HTMLParser, json
from lxml.etree import tostring
from lxml.html.soupparser import fromstring
from lxml.html import HtmlComment
from itertools import izip, cycle
from urlparse import urljoin
from meetingmaps import entmap, uuids
from datetime import date, datetime
from lobbyfacts.data import sl, etl_engine, live_engine
import logging
import sqlalchemy

from lobbyfacts.model import Representative, Meeting, MeetingDeregistered
from lobbyfacts.core import db, app

log = logging.getLogger(__name__)

HEADERS =  { 'User-agent': 'lobbyfacts/1.2' }

mainurl="http://ec.europa.eu/transparencyinitiative/meetings/meeting.do?host=595cf53f-c018-4fc8-afa0-9d66c289795c&d-6679426-p="
trurl="http://ec.europa.eu/transparencyregister/public/consultation/displaylobbyist.do?id="
tregidre=re.compile('[0-9]{8,12}-[0-9]{2}')

h = HTMLParser.HTMLParser()

def unws(obj):
    if isinstance(obj, list):
        obj = u''.join(obj)
    return u' '.join(unicode(obj).split())

def fetch_raw(url, retries=5, ignore=[], params=None):
    try:
        if params:
            r=requests.POST(url, params=params, headers=HEADERS)
        else:
            r=requests.get(url, headers=HEADERS)
    except (requests.exceptions.ConnectionError, requests.exceptions.Timeout), e:
        if e == requests.exceptions.Timeout:
            retries = min(retries, 1)
        if retries>0:
            time.sleep(4*(6-retries))
            f=fetch_raw(url, retries-1, ignore=ignore, params=params)
        else:
            raise ValueError("failed to fetch %s" % url)
    if r.status_code >= 400 and r.status_code not in [504, 502]+ignore:
        print >>sys.stderr, "[!] %d %s" % (r.status_code, url)
        r.raise_for_status()
    return r.text

def fetch(url, retries=5, ignore=[], params=None):
    f = fetch_raw(url, retries, ignore, params)
    return fromstring(f)

host2portfolio = { u"President Jean-Claude Juncker" : "Presidency",
                   u"Commissioner Günther Oettinger":"Digital Economy",
                   u"Commissioner Miguel Arias Cañete":"Climate & Energy",
                   u"Commissioner Cecilia Malmström":"Trade",
                   u"Commissioner Jonathan Hill":"Financial Markets",
                   u"Commissioner Johannes Hahn":"Neighbourhood",
                   u"Vice-President Jyrki Katainen" : "Jobs & Growth",
                   u"Commissioner Christos Stylianides":"Humanitarian Aid",
                   u"Commissioner Corina Crețu":"Regional Policy",
                   u"Commissioner Dimitris Avramopoulos":"Home Affairs",
                   u"First Vice-President Frans Timmermans":"Better Regulation",
                   u"Commissioner Phil Hogan":"Agriculture",
                   u"Commissioner Violeta Bulc":"Transport",
                   u"Commissioner Elżbieta Bieńkowska": "Internal Market",
                   u"Vice-President Kristalina Georgieva": "Budget",
                   u"Vice-President Andrus Ansip":"Digital Single Market",
                   u"Vice-President Maroš Šefčovič":"Energy Union",
                   u"Commissioner Tibor Navracsics":"Education",
                   u"Commissioner Karmenu Vella":"Environment",
                   u"High Representative / Vice-President Federica Mogherini":"External Action",
                   u"Commissioner Vytenis Andriukaitis":"Health",
                   u"Commissioner Neven Mimica":"Development",
                   u"Vice-President Valdis Dombrovskis":"Euro",
                   u"Commissioner Pierre Moscovici":"Economics & Tax",
                   u"Commissioner Věra Jourová":"Justice",
                   u"Commissioner Marianne Thyssen":"Employment",
                   u"Commissioner Carlos Moedas":"Research",
                   u"Commissioner":"Financial Markets",
                   u"Commissioner Margrethe Vestager":"Competition",
}

prefix = 'Meetings of '
prefix2 = 'Cabinet members of '
postfix = ' with organisations and self-employed individuals'
urlroot = 'http://ec.europa.eu/transparencyinitiative/meetings/'

# yields (url, org, title) where
#   url = the url to the table of meetings of a commissioner
#   org = the workgroup that the commissioner is responsible for
#   title = the name of the commissioner
#
# this function also traverses the agendas of predecessors of the commissioners
def get_urls():
    # uuids contains a dictionary of URLs for meetings
    for _, url in uuids:
        root = fetch(url)
        title = unws(' '.join(root.xpath('//h3//text()')))[len(prefix):-len(postfix)]
        crs = root.xpath('//a[contains(@class,"breadcrumb-segment-last")]//text()')
        if crs[0]=='The Commissioners':
            if title.startswith(prefix2):
                org = host2portfolio[title[len(prefix2):]]
            else:
            	try:
                	org = host2portfolio[title]
                except:
                	org = u""
                	print url
        elif crs[0]=='DGs':
            org = crs[1]
            if org == 'Secretariat-General > What we do':
                org = 'Secretariat-General'
        else:
            raise ValueError("uknown crs[0]")
        #print ('\t'.join((url, org, title))).encode('utf8')
        yield (url, org, title)

        predurl = root.xpath('//a[text()="List of predecessors"]')
        if len(predurl)==1:
            # get predecessors urls
            predroot = fetch(urljoin(url, predurl[0].get('href')))
            for pred in predroot.xpath('//div[@id="titlerefpage"]/following-sibling::a'):
                if pred.xpath('./text()')[0].startswith('Back to '):
                    break
                url = "%s%s" % (urlroot, pred.get('href'))
                predroot=fetch(url)
                title = unws(' '.join(predroot.xpath('//h3//text()')))[len(prefix):-len(postfix)]
                crs = predroot.xpath('//a[contains(@class,"breadcrumb-segment-last")]//text()')
                if crs[0]=='The Commissioners':
                    if title.startswith(prefix2):
                        org = host2portfolio[title[len(prefix2):]]
                    else:
                        org = host2portfolio[title]
                elif crs[0]=='DGs':
                    org = crs[1]
                    if org == 'Secretariat-General > What we do':
                        org = 'Secretariat-General'
                else:
                    raise ValueError("uknown crs[0]")
                #print ('\t'.join((url, org, title))).encode('utf8')
                yield (url, org, title)
        elif len(predurl)>1:
            print >>sys.stderr, "wtf", predurl

def get_ents(nodes):
    ents = []
    for lm in nodes:
        if not isinstance(lm, HtmlComment) or trurl not in unicode(lm):
            continue
        lm = unws(lm)
        name = lm[:-8]  # '</a> -->'
        start = lm.index(trurl) + len(trurl)
        end = start+lm[start:].index('"')
        tregid = lm[start:end]
        name = h.unescape(name[end+2:].strip())
        if not name: continue
        if name in entmap:
            name, tregid = entmap[name]
        elif not tregidre.match(tregid):
            tregid = 'unregistered'
            print >>sys.stderr, '[!] unregistered?', name.encode('utf8')
        ents.append((name, tregid))
    return ents

def scrape(url, title, org):
    while True:
        try: root = fetch(url)
        except:
            print >>sys.stderr, 'failed to fetch url', sys.exc_info(), url
            break
        for row in root.xpath('//table[@id="listMeetingsTable"]/tbody/tr'):
            fields = row.xpath('.//td')
            if len(fields) == 5:
                name = unws(fields[0].xpath('.//text()'))
                date = unws(fields[1].xpath('.//text()'))
                location = unws(fields[2].xpath('.//text()'))
                entities = get_ents(fields[3])
                subject = unws(fields[4].xpath('.//text()'))
            elif len(fields) == 4:
                name = unws(title)
                date = unws(fields[0].xpath('.//text()'))
                location = unws(fields[1].xpath('.//text()'))
                entities = get_ents(fields[2])
                subject = unws(fields[3].xpath('.//text()'))
            else:
                print >>sys.stderr, "fields not 4-5, wtf"
                raise
            # calculate uniq meeting id
            meetid = hashlib.md5('\0' * 16)
            for lm in (name, date, location, '\0'.join('\1'.join(e) for e in entities), subject, org):
                meetid = hashlib.md5(meetid.digest()+lm.encode('utf8'))
            meetid = meetid.hexdigest()

            for entity, entity_id in entities:
            	if entity_id == '002977215945-85':
            		print url
                yield {'name': name,
                       'status': 'active',
                       'meetid': meetid,
                       'date': date,
                       'location': location,
                       'subject': subject,
                       'identification_code': entity_id,
                       'representative': entity,
                       'eu_representative': org}

        try: url=urljoin(mainurl, root.xpath('//a/img[@alt="Next"]/..')[0].attrib['href'])
        except: break

def load_meeting(engine, meet):
    # rename: id, name, org
    meet['id'] = meet['meetid']

    del meet['meetid']
    meet['ec_representative'] = meet['name']
    del meet['name']
    meet['ec_org'] = meet['eu_representative']
    del meet['eu_representative']

    # handle cancelled
    meet['cancelled']=False
    if 'Cancelled' in meet['date']:
        meet['cancelled']=True
        meet['date']=meet['date'].split()[0]

    # handle date
    meet['date'] = datetime.strptime(meet['date'],"%d/%m/%Y")

    meeting = Meeting.by_id(meet['id'])
    if meeting is None:
    	print meet['id']
        if meet['identification_code'] == 'unregistered':
            meet['unregistered']=meet['representative']
        meeting = Meeting.create(meet)

    else:
        if meet['identification_code'] == 'unregistered':
            if meeting.unregistered:
                meet['unregistered']='; '.join(sorted(set(meeting.unregistered.split('; ')+[meet['representative']])))
            else:
                meet['unregistered']=meet['representative']
        meeting.update(meet)

    if  meet['identification_code'] != 'unregistered':
        rep = Representative.by_identification_code(meet['identification_code'])
        if rep is None:
            print "could not match", meet['identification_code']
            dereg_ = {'name': meet['representative'],
                      'identification_code': meet['identification_code'],
                      'status': meet['status'],
                      'meeting_id': meeting.id}
            if meeting.deregistered:
                dereg = MeetingDeregistered.by_midc(meeting,meet['identification_code'])
                if not dereg:
                    dereg = MeetingDeregistered.create(dereg_)
                    meeting.deregistered.append(dereg)
                    print 'appended', meeting.unregistered
                else:
                    dereg.update(dereg_)
            else:
                dereg = MeetingDeregistered.create(dereg_)
                meeting.deregistered.append(dereg)

        else:
			if meet['identification_code'] == '002977215945-85':
				print "\o/ match", meet['representative'], meet['identification_code'], meet['id'], meet['status']
			if rep not in meeting.participants:
				meeting.participants.append(rep)

    db.session.commit()

def extract(engine):
    table = sl.get_table(engine, 'meeting')
    try:
    	a = "ok"
        #sl.update(engine, 'meeting', {}, {'status': 'inactive'}, ensure=False)
        #sl.update(engine, 'meeting_participants', {}, {'status': 'inactive'}, ensure=False)
    except sqlalchemy.exc.CompileError:
        pass

    i=0
    for url, org, title in get_urls():
        for meeting in scrape(url, title, org):
            load_meeting(engine,meeting)
            #sl.upsert(engine, table, meeting, ['meetid', 'identification_code'])
            i+=1
            if i % 100 == 0:
                log.info("Extracted: %s...", i)

def extract_new():
    engine = live_engine()
    extract(engine)

def external_url_handler(error, endpoint, values):
    return ''

if __name__ == '__main__':
    app.url_build_error_handlers.append(external_url_handler)
    ctx = app.test_request_context()
    ctx.push()
    global request
    request = app.preprocess_request()
    engine = live_engine()

    reps = Representative.query.all()
    i = 0
    for rep in reps:
        if rep.entity is not None:
            rep.entity.update_index()
        i = i+1
        if i % 100 == 0:
            print i

    db.session.commit()
    print "bye"


    # url = "http://ec.europa.eu/transparencyinitiative/meetings/meeting.do?host=375218b1-5b9b-4f1f-8701-cda656622c9a"
    # f = fetch_raw(url, 5, [], None)
    # f = f.replace(" -- "," - ")
    # f = f.replace(" --<"," -<")
    # s = fromstring(f)
    #
    # app.url_build_error_handlers.append(external_url_handler)
    # ctx = app.test_request_context()
    # ctx.push()
    # global request
    # request = app.preprocess_request()
    # engine = live_engine()
    # if len(sys.argv)<2:
    #     # extract current
    #     extract(engine)
    # else:
    #     # extract from file
    #     with open(sys.argv[1],'r') as fd:
    #         extract_data(engine, fd.read().decode('utf-8'))
