from datetime import datetime
from datetime import timedelta
from lxml import etree
from pprint import pprint
import logging

import requests, sys
from lobbyfacts.data import sl, etl_engine, live_engine
import sqlalchemy

from lobbyfacts.data.load.util import upsert_person, upsert_organisation, upsert_entity, upsert_tag, upsert_contact, upsert_representative
from lobbyfacts.model import Category, Country
from lobbyfacts.core import db
from lobbyfacts.core import app
from lobbyfacts.model.representative import *
from lobbyfacts.data.lib.countries import country_by_name
from lobbyfacts.data.load.util import to_integer

log = logging.getLogger(__name__)

URL = 'http://ec.europa.eu/transparencyregister/public/consultation/statistics.do?action=getLobbyistsXml&fileType=NEW'
NS2 = "{http://www.w3.org/1999/xlink}"
NS = "{http://intragate.ec.europa.eu/transparencyregister/intws/20141104}"
SI = "{http://www.w3.org/2001/XMLSchema-instance}"

def upsert_category(id, name, parent=None):
    if name == 'II - In-house lobbyists and trade/business/professional associations':
        name = 'II - In-house lobbyists and trade/professional associations'
    elif name == 'Other organisations':
        name = 'Other in house lobbyists'
    elif name == 'Trade and business associations':
        name = 'Trade and business organisations'
    data = {'name': name, 'parent': parent}
    category = Category.by_name(name)
    if category is None:
    	log.info('creating new category: not good')
        category = Category.create(data)
    else:
    	data['id'] = category.id
        category.update(data)
    db.session.commit()
    return category

def dateconv(ds):
    return datetime.strptime(ds.split("+")[0].strip(), "%Y-%m-%dT%H:%M:%S.%f")

def intconv(val):
    return val

def parse_rep(rep_el, xml_extract_date):

    rep = {}
    rep['xml_extract_date'] = xml_extract_date
    rep['state_date'] = xml_extract_date
    rep['identification_code'] = rep_el.findtext(NS + 'identificationCode')
    rep['status'] = rep_el.findtext(NS + 'status')
    rep['registration_date'] = dateconv(rep_el.findtext(NS + 'registrationDate'))
    rep['last_update_date'] = dateconv(rep_el.findtext(NS + 'lastUpdateDate'))
    rep['legal_status'] = rep_el.findtext(NS + 'legalStatus')
    rep['acronym'] = rep_el.findtext(NS + 'acronym')
    latin_name = rep_el.findtext('.//' + NS + 'nameInLatinAlphabet')
    if latin_name:
        rep['original_name'] = latin_name
        rep['native_name'] = rep_el.findtext('.//' + NS + 'originalName')
    else:
        rep['original_name'] = rep_el.findtext('.//' + NS + 'originalName')

    el = rep_el.find(NS + 'webSiteURL')
    rep['web_site_url'] = el.get(NS2 + 'href') if el is not None else None
    rep['main_category'] = rep_el.findtext('.//' + NS + 'mainCategory')
    rep['sub_category'] = rep_el.findtext('.//' + NS + 'subCategory')

    legal = {}
    legal['title'] = rep_el.findtext(NS + 'legalResp/' + NS + 'title')
    legal['first_name'] = rep_el.findtext(NS + 'legalResp/' + NS +
            'firstName')
    legal['last_name'] = rep_el.findtext(NS + 'legalResp/' + NS +
            'lastName')
    legal['position'] = rep_el.findtext(NS + 'legalResp/' + NS +
            'position')
    rep['legal_person'] = legal

    head = {}
    head['title'] = rep_el.findtext(NS + 'euRelationsResp/' + NS + 'title')
    head['first_name'] = rep_el.findtext(NS + 'euRelationsResp/' + NS +
            'firstName')
    head['last_name'] = rep_el.findtext(NS + 'euRelationsResp/' + NS +
            'lastName')
    head['position'] = rep_el.findtext(NS + 'euRelationsResp/' + NS +
            'position')
    rep['head_person'] = head

    for tag in ['contactDetails/', 'contactDetailsBelgium/']:
        contact={}
        contact['street'] = ' '.join((
            rep_el.findtext(NS + tag + NS + 'addressline1') or '',
            rep_el.findtext(NS + tag + NS + 'addressline2') or '',
            rep_el.findtext(NS + tag + NS + 'addressline2') or ''))
        contact['postbox'] = rep_el.findtext(NS + tag + NS + 'postBox')
        contact['post_code'] = rep_el.findtext(NS + tag + NS + 'postCode')
        contact['town'] = rep_el.findtext(NS + tag + NS + 'town')
        contact['country'] = rep_el.findtext(NS + tag + NS + 'country')
        contact['indic_phone'] = rep_el.findtext(NS + tag +'/'+NS+'phone/'+ NS + 'indicPhone')
        contact['phone'] = rep_el.findtext(NS + tag +'/'+NS+'phone/'+ NS + 'phoneNumber')
        if tag == 'contactDetails/':
            contact['type']='head'
            rep['head_contact']=contact
        elif tag == 'contactDetailsBelgium/':
            contact['type']='belgium'
            rep['be_contact']=contact
        else:
            raise ValueError(tag) # should never happen

    rep['goals'] = rep_el.findtext(NS + 'goals')
    act_cats = [
        ('activityConsultCommittees', 'activity_consult_committees'),
        ('activityEuLegislative','activity_eu_legislative'),
        ('activityExpertGroups','activity_expert_groups'),
        ('activityHighLevelGroups','activity_high_level_groups'),
        ('activityIndustryForums','activity_industry_forums'),
        ('activityInterGroups','activity_inter_groups'),
        ('activityOther','activity_other'),
        ('activityRelevantComm','activity_relevant_comm')]
    for lm, k in act_cats:
        rep[k] = rep_el.findtext('.//' + NS + 'activities/' + NS + lm)

    rep['code_of_conduct'] = rep_el.findtext('.//' + NS + 'codeOfConduct')
    # todo remove ohercodeofconduct in a few weeks time
    rep['other_code_of_conduct'] = rep_el.findtext('.//' + NS + 'oherCodeOfConduct') or rep_el.findtext('.//' + NS + 'otherCodeOfConduct')

    rep['members_25'] = rep_el.findtext('.//' + NS + 'members25Percent')
    rep['members_50'] = rep_el.findtext('.//' + NS + 'members50Percent')
    rep['members_75'] = rep_el.findtext('.//' + NS + 'members75Percent')
    rep['members_100'] = rep_el.findtext('.//' + NS + 'members100Percent')
    rep['members_fte'] = rep_el.findtext('.//' + NS + 'membersFTE')
    rep['info_members'] = rep_el.findtext('.//' + NS + 'infoMembers')
    rep['members'] = to_integer(rep_el.findtext('.//' + NS + 'members/' + NS + 'members'))

    rep['action_fields'] = []
    for field in rep_el.findall('.//' + NS + 'actionField/' + NS +
            'actionField'):
        rep['action_fields'].append(field.text)
    rep['interests'] = []
    for interest in rep_el.findall('.//' + NS + 'interest/' + NS +
            'name'):
        rep['interests'].append(interest.text)
    rep['number_of_natural_persons'] = intconv(rep_el.findtext('.//' + NS + 'structure/' + NS
            + 'numberOfNaturalPersons'))
    rep['structure_members'] = rep_el.findtext('.//' + NS + 'structure/' + NS
            + 'structureMembers')
    rep['networking'] = rep_el.findtext('.//' + NS + 'structure/' + NS + 'networking')
    rep['country_of_members'] = []
    el = rep_el.find(NS + 'structure/' + NS + 'countries')
    if el is not None:
        for country in el.findall('.//' + NS + 'country'):
            rep['country_of_members'].append(country.text)
    rep['organisations'] = []
    el = rep_el.find(NS + 'structure/' + NS + 'organisations')
    if el is not None:
        for org_el in el.findall(NS + 'organisation'):
            org = {}
            org['name'] = org_el.findtext(NS + 'name')
            org['number_of_members'] = org_el.findtext(NS + 'numberOfMembers')
            rep['organisations'].append(org)
            state['organisations'].append(org)

    fd_el = rep_el.find(NS + 'financialData')
    fd = {}
    rep['new_organisation'] = fd_el.findtext(NS + 'newOrganisation')
    try:
        fd['start_date'] = dateconv(fd_el.findtext(NS + 'startDate'))
    except AttributeError:
        if rep['new_organisation'] != 'true':
            print >>sys.stderr, '[x] missing financial data, check out:', rep['identification_code']
        rep['fd'] = fd
        return rep
    fd['end_date'] = dateconv(fd_el.findtext(NS + 'endDate'))
    fd['eur_sources_procurement'] = intconv(fd_el.findtext(NS + 'eurSourcesProcurement'))
    fd['eur_sources_procurement_src'] = fd_el.findtext(NS + 'eurSourcesProcurementSrc')
    fd['eur_sources_grants'] = intconv(fd_el.findtext(NS + 'eurSourcesGrants'))
    fd['eur_sources_grants_src'] = fd_el.findtext(NS + 'eurSourcesGrantsSrc')
    fi = fd_el.find(NS + 'financialInformation')
    fd['type'] = fi.get(SI + 'type')
    fd['no_clients'] = fi.findtext(NS + 'noClientsInfo')
    #import ipdb; ipdb.set_trace()
    fd['total_budget'] = intconv(fi.findtext('.//' + NS +
        'totalBudget'))
    fd['public_financing_total'] = intconv(fi.findtext('.//' + NS +
        'totalPublicFinancing'))
    fd['public_financing_national'] = intconv(fi.findtext('.//' + NS +
        'nationalSources'))
    fd['public_financing_infranational'] = intconv(fi.findtext('.//' + NS +
        'infranationalSources'))
    cps = fi.find('.//' + NS + 'customisedPublicSources')
    fd['public_customized'] = []
    if cps is not None:
        for src_el in cps.findall('.//' + NS + 'customisedSource'):
            src = {}
            src['name'] = src_el.findtext(NS + 'name')
            src['amount'] = intconv(src_el.findtext(NS + 'amount'))
            fd['public_customized'].append(src)
    fd['other_sources_total'] = intconv(fi.findtext('.//' + NS +
        'totalOtherSources'))
    fd['other_sources_donation'] = intconv(fi.findtext('.//' + NS +
        'donation'))
    fd['other_sources_contributions'] = intconv(fi.findtext('.//' + NS +
        'contributions'))
    # TODO customisedOther
    cps = fi.find('.//' + NS + 'customisedOther')
    fd['other_customized'] = []
    if cps is not None:
        for src_el in cps.findall('.//' + NS + 'customisedSource'):
            src = {}
            src['name'] = src_el.findtext(NS + 'name')
            src['amount'] = intconv(src_el.findtext(NS + 'amount'))
            fd['other_customized'].append(src)

    fd['direct_rep_costs_min'] = intconv(fi.findtext('.//' + NS +
        'directRepresentationCosts//' + NS + 'min'))
    fd['direct_rep_costs_max'] = intconv(fi.findtext('.//' + NS +
        'directRepresentationCosts//' + NS + 'max'))
    fd['cost_min'] = intconv(fi.findtext('.//' + NS +
        'cost//' + NS + 'min'))
    fd['cost_max'] = intconv(fi.findtext('.//' + NS +
        'cost//' + NS + 'max'))
    fd['cost_absolute'] = intconv(fi.findtext('.//' + NS +
        'cost//' + NS + 'absoluteCost'))
    fd['turnover_min'] = intconv(fi.findtext('.//' + NS +
        'turnover//' + NS + 'min'))
    fd['turnover_max'] = intconv(fi.findtext('.//' + NS +
        'turnover//' + NS + 'max'))
    fd['turnover_absolute'] = intconv(fi.findtext('.//' + NS +
        'turnover//' + NS + 'absoluteAmount'))

    fd['turnover_breakdown'] = []
    for tag in ['turnoverBreakdown', 'newTurnoverBreakdown']:
        tb = fi.find(NS + tag)
        newbd = (tag == 'newTurnoverBreakdown')
        if tb is not None:
            for range_ in tb.findall(NS + 'customersGroupsInAbsoluteRange'):
                max_ = range_.findtext('.//' + NS + 'max')
                min_ = range_.findtext('.//' + NS + 'min')
                for customer in range_.findall('.//' + NS + 'customer'):
                    fd['turnover_breakdown'].append({
                        'name': customer.findtext(NS + 'name'),
                        'new': newbd,
                        'min': intconv(min_),
                        'max': intconv(max_)
                        })
            for range_ in tb.findall(NS + 'customersGroupsInPercentageRange'):
                # FIXME: I hate political compromises going into DB design
                # so directly.
                max_ = range_.findtext('.//' + NS + 'max')
                if max_:
                    max_ = float(max_) / 100.0 * \
                            float(fd['turnover_absolute'] or
                                  fd['turnover_max'] or fd['turnover_min'])
                min_ = range_.findtext('.//' + NS + 'min')
                if min_:
                    min_ = float(min_) / 100.0 * \
                            float(fd['turnover_absolute'] or
                                  fd['turnover_min'] or fd['turnover_max'])
                for customer in range_.findall('.//' + NS + 'customer'):
                    fd['turnover_breakdown'].append({
                        'name': customer.findtext(NS + 'name'),
                        'new': newbd,
                        'min': intconv(min_),
                        'max': intconv(max_)
                        })
    fd['other_financial_information'] = fd_el.findtext(NS + 'otherFinancialInformation')
    rep['fd'] = fd

    return rep

def load_person(person, role, childBase, engine):
    table = sl.get_table(engine, 'person')
    person_ = childBase.copy()
    person_.update(person)
    person_['role'] = role
    person_['name'] = ' '.join((person['title'] or '',
                                person['first_name'] or '',
                                person['last_name'] or ''))
    sl.upsert(engine, table, person_, ['representative_etl_id',
                                       'role',
                                       'name'])


def load_contact(contact, childBase, engine):
    if contact == {}: return
    table = sl.get_table(engine, 'contact')
    contact_ = childBase.copy()
    contact_.update(contact)
    sl.upsert(engine, table, contact_, ['representative_etl_id', 'country', 'type'])

def load_finances(financialData, childBase, engine):
    if financialData == {}: return
    etlId = '%s//%s' % (financialData['start_date'].isoformat(),
                        financialData['end_date'].isoformat())

    financial_sources = \
        [(s, 'other') for s in financialData.pop("other_customized")] + \
        [(s, 'public') for s in financialData.pop("public_customized")]
    for financial_source, type_ in financial_sources:
        financial_source['type'] = type_
        financial_source['financial_data_etl_id'] = etlId
        financial_source.update(childBase)
        sl.upsert(engine, sl.get_table(engine, 'financial_data_custom_source'),
                  financial_source, ['representative_etl_id',
                      'financial_data_etl_id', 'type', 'name'])

    for turnover in financialData.pop("turnover_breakdown"):
        turnover['financial_data_etl_id'] = etlId
        turnover['name'] = turnover['name'].strip()
        turnover.update(childBase)
        sl.upsert(engine, sl.get_table(engine, 'financial_data_turnover'),
                  turnover, ['representative_etl_id', 'financial_data_etl_id',
                             'name'])

    financialData['etl_id'] = etlId
    financialData.update(childBase)
    sl.upsert(engine, sl.get_table(engine, 'financial_data'),
              financialData, ['representative_etl_id', 'etl_id'])
    #pprint(financialData)

# state: dictionary created by parsing the XML
# rep: representative object that was saved to the database
def load_rep_state(rep, state, engine):

    state['rep_id'] = rep.id
    person_data = state['head_person']
    person_list = [person_data['title'],person_data['first_name'],person_data['last_name']]
    person_data['name'] = u' '.join(item for item in person_list if item)
    person = upsert_person(person_data)
    person.entity.update_index()

    state['head'] = person

    person_data = state['legal_person']
    person_list = [person_data['title'],person_data['first_name'],person_data['last_name']]
    person_data['name'] = u' '.join(item for item in person_list if item)


    person = upsert_person(person_data)
    person.entity.update_index()
    state['legal'] = person

    contact_data = state['head_contact']
    contact_ = {
            'town': contact_data['town'],
            'street': contact_data['street'],
            'post_code': contact_data['post_code'],
            'phone': " ".join((contact_data.get('indic_phone') or '', contact_data.get('phone') or '')).strip(),
            }
    cco = country_by_name(contact_data['country'])
    contact_['country'] = Country.by_code(cco['iso2'])
    contact = upsert_contact(contact_)


    state['head_office'] = contact

    if(state['be_contact']['country'] is not None):
        contact_data = state['be_contact']
        contact_ = {
            'town': contact_data['town'],
            'street': contact_data['street'],
            'post_code': contact_data['post_code'],
            'phone': " ".join((contact_data.get('indic_phone') or '', contact_data.get('phone') or '')).strip(),
            }
        cco = country_by_name(contact_data['country'])
        contact_['country'] = Country.by_code(cco['iso2'])
        contact = upsert_contact(contact_)
        state['be_office'] = contact

    log.info("creating state...");
    rep_state = RepresentativeState.create(state)

    if state == {}: return

    for action_ in state['action_fields']:

		af = ActionField.by_action(action_)
		if af is None:
			af = ActionField.create({'action': action_})

		adata = {'state': rep_state,
				 'action': af}

		am = StateAssociatedAction.create(adata)

    for interest_ in state['interests']:
        i = Interest.by_interest(interest_)
        if i is None:
            i = Interest.create({'interest': interest_})

        adata = {'state': rep_state,
                 'interest': i}

        ai = StateAssociatedInterest.create(adata)

    fd = state['fd']
    fd['representative'] = rep
    state_financial_data = StateFinancialData.create(fd)
    rep_state.state_financial_data = state_financial_data

    try:
        for ft in fd['turnover_breakdown']:
            ft['state_financial_data'] = state_financial_data

            ft['entity'] = upsert_entity(ft['name'])
            ft['min'] = to_integer(ft.get('min'))
            ft['max'] = to_integer(ft.get('max'))
            state_financial_turnover = StateFinancialTurnover.create(ft)
    except KeyError:
        log.info("No turnover breakdown...")

    try:
        for src_ in fd['other_customized']:
            src_['state_financial_data'] = state_financial_data
            src_['amount'] = to_integer(src_.get('amount'))
            src_['type'] = 'other'
            src = StateCustomIncome.create(src_)
    except KeyError:
        log.info("No other customized sources...")

    try:
        for src_ in fd['public_customized']:
            src_['state_financial_data'] = state_financial_data
            src_['amount'] = to_integer(src_.get('amount'))
            src_['type'] = 'public'
            src = StateCustomIncome.create(src_)
    except KeyError:
        log.info("No public customized sources...")

    try:
        for org in state['organisations']:
            e = upsert_entity(org['name'])
            e.update_index()
            org['entity'] = e
            os = OrganisationState.by_all(org)
            if os is None:
                os = OrganisationState.create(org)
            osm = {}
            osm['organisation_state'] = os
            osm['representative_state'] = rep_state
            OrganisationStateMembership.create(osm)
    except KeyError:
        log.info("caught ORG key error...")


def load_rep(rep, engine):
    if not rep['original_name']:
        log.error("Unnamed representative: %r", rep)
        return

    entity = upsert_entity(rep.get('canonical_name'),
                name=rep.get('original_name'),
                suffix=rep.get('name_suffix'),
                acronym=rep.get('acronym'))
    assert entity is not None, entity
    assert entity.id is not None, entity
    rep['entity'] = entity
    rep['status'] = 'active'
    if rep.get('main_category'):
        main_category = upsert_category(rep.get('main_category_id'),
                                        rep.get('main_category'))
        rep['main_category'] = main_category
        rep['sub_category'] = upsert_category(rep.get('sub_category_id'),
                                              rep.get('sub_category'),
                                              main_category)
    rep_obj = upsert_representative(rep)
    entity.update_index()
    load_rep_state(rep_obj, rep, engine)
    return rep_obj

def parse(doc):
    tel = doc.find('.//' + NS + 'exportDate');
    xml_extract_date = dateconv(tel.text)
    for rep_el in doc.findall('.//' + NS + 'interestRepresentative'):
        yield parse_rep(rep_el, xml_extract_date)

def extract_data(engine, data):
    line = ""
    # crucialfile keeps track of the identification codes of representatives to
    # see when they appear or disappear from the XMLs.
    try:
        f = open('/home/lobbyfacts/api-newxml/bin/crucialfile','r')
        line = f.readline()
        f.close()
    except:
        log.error("something went wrong reading the file. cancelling import!")
        return

    doc = etree.fromstring(data.encode('utf-8'))
    tel = doc.find('.//' + NS + 'exportDate')

    xml_extract_date = dateconv(tel.text)

    # the date of the previously extracted XML is in the database
    ld_obj = LastDate.get_lastdate()

    # check whether the date of this XML is later than that of the previous import
    if ld_obj is None:
        old_date = None
        ld = {}
        ld['lastdate'] = xml_extract_date
        ld_obj = LastDate.create(ld)
        db.session.commit()
    elif ld_obj.lastdate >= xml_extract_date:
    	log.info('duplicate XML or dubious export date. skipping. exportDate: ' + str(xml_extract_date))
    	return
    else:
        old_date = ld_obj.lastdate
        ld = {}
        ld['lastdate'] = xml_extract_date
        ld_obj.update(ld)

    # only reps in current XML should be active
    sl.update(engine, 'representative', {}, {'status': 'inactive'}, ensure=False)

    rep_list = line.split(",")
    new_rep_set = set()
    done_list = []

    log.info("Extracting registered interests data...")
    repid = 0
    for i, rep in enumerate(parse(doc)):
        # find the representative by identification code.
        # check whether the last update date of this representative signals a
        # relevant change to the data of this representative
        representative = Representative.by_identification_code(rep['identification_code'])


        if representative is not None:
            astate = RepresentativeState.by_date_id(xml_extract_date,representative.id)
            if astate is not None:
                log.info("%s already has this state. skipping.", representative.id)
                continue

        if (representative is None or old_date is None or (rep['last_update_date'] > old_date) or (rep['registration_date'] > old_date)):
            astate = None
            log.info('extract/reginterest.py: modified rep id-code: %s', rep['identification_code'])
            log.info('xml date: %s - update date: %s', str(rep['xml_extract_date']),str(rep['last_update_date']))
            newrep = load_rep(rep,engine)
            newrep.entity.update_index()
            repid = newrep.id
        if representative is not None:
            representative.status = 'active'
            repid = representative.id

        # keep track of (dis)appearances of representatives using the list we
        # extracted from crucialfile (see higher)
        new_rep_set.add(rep['identification_code'])
        ridc = rep['identification_code']
        if(ridc not in rep_list):
            if ridc not in done_list:
                log.info(ridc + " has appeared.")
                _data = {'representative_id':repid, 'identification_code': rep['identification_code'],'date':rep['xml_extract_date'],'disappeared':False}
                rapp = rep_appearance.create(_data)
        else:
            rep_list.remove(rep['identification_code'])
        done_list.append(ridc)

        if i % 50 == 0:
            log.info("Extracted: %s...", i)

    # more tracking of (dis)appearances
    for trep in rep_list:
        if trep != "" and trep is not None:
            _data = {'identification_code': trep,'date':rep['xml_extract_date'],'disappeared':True}
            rapp_id = db.session.execute("select id from rep_appearance where identification_code = :rid and disappeared_date is null",{'rid':trep})
            t_rapp = rapp_id.fetchone()
            db.session.execute("update rep_appearance set disappeared_date = :dd where id = :tid",{'dd':rep['xml_extract_date'],'tid':t_rapp['id']})
            log.info(trep + " has disappeared.")

    newline = ",".join(new_rep_set)

    try:
        f = open('/home/lobbyfacts/api-newxml/bin/crucialfile','w')
        f.write(newline)
        f.close()
    except:
        log.error("something went wrong writing to file")

    db.session.commit()

def extract(engine):
    try:
        sl.update(engine, 'representative', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'contact', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'financial_data', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'financial_data_turnover', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'person', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'organisation', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'accreditation', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'country_of_member', {}, {'status': 'inactive'}, ensure=False)
        # sl.update(engine, 'associated_action', {}, {'status': 'inactive'}, ensure=False)
    except sqlalchemy.exc.CompileError:
        pass

    res = requests.get(URL)
    extract_data(engine, res.content.decode('utf-8'))

def extract_new():
    engine = live_engine()

    res = requests.get(URL)
    extract_data(engine, res.content.decode('utf-8'))

def external_url_handler(error, endpoint, values):
    return ''

if __name__ == '__main__':
    app.url_build_error_handlers.append(external_url_handler)
    ctx = app.test_request_context()
    ctx.push()
    global request
    request = app.preprocess_request()
    engine = live_engine()
    if len(sys.argv)<2:
        # extract current
        extract(engine)
    else:
        # extract from file
        log.info("Processing file " + sys.argv[1])
        with open(sys.argv[1],'r') as fd:
            extract_data(engine, fd.read().decode('utf-8'))
