from lobbyfacts.core import db
from lobbyfacts.model.api import ApiEntityMixIn
from lobbyfacts.model.revision import RevisionedMixIn
from lobbyfacts.model.entity import Entity
from lobbyfacts.model.tag import Tag
from datetime import datetime
from lobbyfacts.model.action import ActionField, AssociatedAction
from lobbyfacts.model.interest import Interest, AssociatedInterest

def to_integer(val):
    if val is None:
        return None
    try:
        return int(float(val))
    except ValueError:
        return None

class Tags(db.Model, ApiEntityMixIn):
    __tablename__ = 'tags'
    representative_id = db.Column('representative_id', db.String(36), db.ForeignKey('representative.id'), primary_key=True)
    tag_id = db.Column('tag_id', db.BigInteger, db.ForeignKey('tag.id'), primary_key=True)

    def as_shallow(self):
        return { 'representative': Representative.by_id(self.representative_id).entity.name,
                 'tags': Tag.by_id(self.tag_id),
                 'representative_id': self.representative_id }

    def as_dict(self):
        return self.as_shallow()

    @classmethod
    def all(cls):
        return db.session.query(cls)

class Contact(db.Model, ApiEntityMixIn, RevisionedMixIn):
    __tablename__ = 'contact'

    id = db.Column(db.BigInteger, primary_key=True)
    town = db.Column(db.Unicode)
    street = db.Column(db.Unicode)
    phone = db.Column(db.Unicode)
    post_code = db.Column(db.Unicode)
    postbox = db.Column(db.Unicode)
    lat = db.Column(db.Float)
    lon = db.Column(db.Float)
    country_id = db.Column(db.BigInteger, db.ForeignKey('country.id'))
    country = db.relationship('Country', backref=db.backref('contacts'))

    @classmethod
    def create(cls, data):
        cls = cls()
        return cls.update(data)

    def update(self, data):
        self.street = data.get('street')
        self.town = data.get('town')
        self.street = data.get('street')
        self.phone = data.get('phone')
        self.post_code = data.get('post_code')
        self.postbox = data.get('postbox')
        self.lon = data.get('lon')
        self.lat = data.get('lat')
        self.country = data.get('country')
        self.updated_at=data.get('updated_at')
        self.deleted_at=data.get('deleted_at')
        db.session.add(self)
        return self

    @classmethod
    def by_id(cls, id):
        return cls.by_attr(cls.id, id)

    @classmethod
    def by_all(cls, data):
        q = db.session.query(cls)
        q = q.filter_by(street=data['street'])
        q = q.filter_by(town=data['town'])
        q = q.filter_by(post_code=data['post_code'])
        q = q.filter_by(phone=data['phone'])
#        q = q.filter_by(number=data['number'])
        return q.first()

    @classmethod
    def all(cls):
        return db.session.query(cls)

    def as_shallow(self):
        d= {
#             'created_at': self.created_at,
#             'updated_at': self.updated_at,
            'town': self.town,
            'street': self.street,
            'phone': self.phone,
            'lon': self.lon,
            'lat': self.lat,
            'post_code': self.post_code,
            'postbox': self.postbox,
            'country': self.country.name,
            }
        if self.id:
            d['id']=self.id
        return d

    def as_dict(self):
        d = self.as_shallow()
        return d

    def __repr__(self):
        return "<Contact(%s)>" % (' '.join((self.street or '',self.town or '',self.post_code or '',self.country.name)))

class Representative(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'representative'

    entity_id = db.Column(db.String(36), db.ForeignKey('entity.id'))

    identification_code = db.Column(db.Unicode)
    native_name = db.Column(db.Unicode)
    goals = db.Column(db.Unicode)

    activity_consult_committees = db.Column(db.Unicode)
    activity_eu_legislative = db.Column(db.Unicode)
    activity_expert_groups = db.Column(db.Unicode)
    activity_high_level_groups = db.Column(db.Unicode)
    activity_industry_forums = db.Column(db.Unicode)
    activity_inter_groups = db.Column(db.Unicode)
    activity_other = db.Column(db.Unicode)
    activity_relevant_comm = db.Column(db.Unicode)

    status = db.Column(db.Unicode)
    networking = db.Column(db.Unicode)
    legal_status = db.Column(db.Unicode)
    code_of_conduct = db.Column(db.Unicode)
    other_code_of_conduct = db.Column(db.Unicode)
    web_site_url = db.Column(db.Unicode)
    info_members = db.Column(db.Unicode)
    structure_members = db.Column(db.Unicode)

    members = db.Column(db.BigInteger, nullable=True)
    members_25 = db.Column(db.BigInteger, nullable=True)
    members_50 = db.Column(db.BigInteger, nullable=True)
    members_75 = db.Column(db.BigInteger, nullable=True)
    members_100 = db.Column(db.BigInteger, nullable=True)
    members_fte = db.Column(db.Float, nullable=True)

    number_of_natural_persons = db.Column(db.BigInteger, nullable=True)

    registration_date = db.Column(db.DateTime)
    last_update_date = db.Column(db.DateTime)

    head_office_id = db.Column(db.BigInteger, db.ForeignKey('contact.id'), nullable=True)
    head_office = db.relationship('Contact',
                                  primaryjoin='Representative.head_office_id==Contact.id',
                                  backref=db.backref('head_offices', uselist=False))
    be_office_id = db.Column(db.BigInteger, db.ForeignKey('contact.id'), nullable=True)
    be_office = db.relationship('Contact',
                                primaryjoin='Representative.be_office_id==Contact.id',
                                backref=db.backref('be_offices', uselist=False))
    # for backward compatibility == nationality of rep
    contact_country_id = db.Column(db.BigInteger, db.ForeignKey('country.id'))

    main_category_id = db.Column(db.BigInteger, db.ForeignKey('category.id'))
    sub_category_id = db.Column(db.BigInteger, db.ForeignKey('category.id'))
    head_id = db.Column(db.Unicode(36), db.ForeignKey('person.id'))
    legal_id = db.Column(db.Unicode(36), db.ForeignKey('person.id'))

    tags = db.relationship("Tag", secondary=Tags.__table__, backref='representatives')

    def update_values(self, data):
        self.entity = data.get('entity')
        assert self.entity is not None, self.entity

        self.identification_code = data.get('identification_code')

        self.native_name = data.get('native_name')
        self.goals = data.get('goals')
        self.status = data.get('status')
        self.activity_consult_committees = data.get('activity_consult_committees')
        self.activity_eu_legislative = data.get('activity_eu_legislative')
        self.activity_expert_groups = data.get('activity_expert_groups')
        self.activity_high_level_groups = data.get('activity_high_level_groups')
        self.activity_industry_forums = data.get('activity_industry_forums')
        self.activity_inter_groups = data.get('activity_inter_groups')
        self.activity_other = data.get('activity_other')
        self.activity_relevant_comm = data.get('activity_relevant_comm')
        self.networking = data.get('networking')
        self.code_of_conduct = data.get('code_of_conduct')
        self.other_code_of_conduct = data.get('other_code_of_conduct')
        self.web_site_url = data.get('web_site_url')
        self.legal_status = data.get('legal_status')

        self.members = data.get('members')
        self.members_25 = data.get('members_25')
        self.members_50 = data.get('members_50')
        self.members_75 = data.get('members_75')
        self.members_100 = data.get('members_100')
        self.members_fte = data.get('members_fte')
        self.info_members = data.get('info_members')
        self.structure_members = data.get('structure_members')
        self.number_of_natural_persons = data.get('number_of_natural_persons')

        self.registration_date = data.get('registration_date')
        self.last_update_date = data.get('last_update_date')

        self.main_category = data.get('main_category')
        self.sub_category = data.get('sub_category')

        self.head = data.get('head')
        self.legal = data.get('legal')

    @classmethod
    def by_identification_code(cls, identification_code):
        return cls.by_attr(cls.identification_code,
                           identification_code)

    @classmethod
    def by_id(cls, id):
        return cls.by_attr(cls.id,
                           id)

    def meetings_sorted(self):
        #mt = self.meetings
        mt = {m for m in self.meetings if m.status == 'active'}
        srtd = sorted(mt, key=lambda meeting: meeting.date,reverse=True)
        return srtd

    def get_clients(self):
        res = []
        for fd in self.state_financial_datas:
            for ft in fd.turnovers:
                res.append(unicode(ft.entity.name))
        return res

    def as_shallow(self):
        d = super(Representative, self).as_dict()

        d.update({
            'uri': self.uri,
            'identification_code': self.identification_code,
            'native_name': self.native_name,
            'goals': self.goals,
            'status': self.status,
            'activity_consult_committees': self.activity_consult_committees,
            'activity_eu_legislative': self.activity_eu_legislative,
            'activity_expert_groups': self.activity_expert_groups,
            'activity_high_level_groups': self.activity_high_level_groups,
            'activity_industry_forums': self.activity_industry_forums,
            'activity_inter_groups': self.activity_inter_groups,
            'activity_other': self.activity_other,
            'activity_relevant_comm': self.activity_relevant_comm,
            'networking': self.networking,
            'code_of_conduct': self.code_of_conduct,
            'other_code_of_conduct': self.other_code_of_conduct,
            'web_site_url': self.web_site_url,
            'legal_status': self.legal_status,
            'members': self.members,
            'members_25': self.members_25,
            'members_50': self.members_50,
            'members_75': self.members_75,
            'members_100': self.members_100,
            'members_fte': self.members_fte,
            'structure_members': self.structure_members,
            'info_members': self.info_members,
            'number_of_natural_persons': self.number_of_natural_persons,
            # 'financial_data': [fd.as_shallow(turnovers=True) for fd in self.financial_datas],
            'clients' : self.get_clients(),
            'registration_date': self.registration_date,
            'last_update_date': self.last_update_date,
            #'meetings': [m.as_dict() for m in self.meetings_sorted()],
            # 'accreditations': [a.as_dict(representative=False) for a in self.accreditations],
            #'accreditations': accs,
            })
        if self.entity:
            d['entity']=self.entity_id
            d['name']=self.entity.name
            d['acronym']=self.entity.acronym
        if self.contact_country:
            d['contact_country']=self.contact_country_id
        if self.head_office:
            d['head_office_phone']=self.head_office.phone
            d['head_office_street']=self.head_office.street
            d['head_office_town']=self.head_office.town
            d['head_office_post_code']=self.head_office.post_code
            d['head_office_postbox']=self.head_office.postbox
            d['head_office_lat']=self.head_office.lat
            d['head_office_lon']=self.head_office.lon
            d['head_office_country']=self.head_office.country.name
        if self.main_category:
            d['main_category']=self.main_category_id
            d['main_category_title']=self.main_category.name
        if self.sub_category:
            d['sub_category']=self.sub_category_id
            d['sub_category_title']=self.sub_category.name
        if self.head:
            d['head']=self.head_id
        if self.legal:
            d['legal']=self.legal_id
        if self.be_office:
            d['be_office_phone']=self.be_office.phone
            d['be_office_street']=self.be_office.street
            d['be_office_town']=self.be_office.town
            d['be_office_post_code']=self.be_office.post_code
            d['be_office_postbox']=self.be_office.postbox
            d['be_office_lat']=self.be_office.lat
            d['be_office_lon']=self.be_office.lon
            d['be_office_country']=self.be_office.country.name
        return d

    def as_dict(self):
        # shallow but without flattened office info
        d = {k:v for k,v in self.as_shallow().items() if not k.startswith('head_office') and not k.startswith('be_office')}
        d.update({
            'entity': self.entity.as_shallow() if self.entity else None,
            'contact_country': self.contact_country.as_shallow() if self.contact_country else None,
            'head_office': self.head_office.as_shallow() if self.head_office else None,
            'be_office': self.be_office.as_shallow() if self.be_office else None,
            'main_category': self.main_category.as_shallow() if self.main_category else None,
            'sub_category': self.sub_category.as_shallow() if self.sub_category else None,
            'head': self.head.as_shallow() if self.head else None,
            'legal': self.legal.as_shallow() if self.legal else None,
            #'financial_data': [fd.as_shallow(turnovers=True) for fd in self.financial_datas],
            'organisation_memberships': [om.as_dict(representative=False) for om in self.organisation_memberships],
            'action_fields': [af.action for af in self.action_fields],
            'interests': [i.interest for i in self.interests],
            'accreditations': [a.as_name() for a in self.accreditations if a.status == 'active'],
            'meetings': [m.as_dict() for m in self.meetings_sorted()],
            'tags': [t.tag for t in self.tags]
            })
        return d

    def cascade_delete(self):
        for a in self.accreditations:
            a.delete()
        for om in self.organisation_memberships:
            om.delete()
        for fd in self.financial_datas:
            fd.delete()

    def __repr__(self):
        return "<Representative(%s,%r)>" % (self.id, self.entity)

Entity.representative = db.relationship(Representative,
        uselist=False, backref=db.backref('entity'))

Representative.action_fields = db.relationship(AssociatedAction,
    uselist=True,
    backref=db.backref('representative',
        # lazy='dynamic',
        ))

Representative.interests = db.relationship(AssociatedInterest,
    uselist=True,
    backref=db.backref('representative',
        # lazy='dynamic',
        ))

class RepresentativeState(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'representative_state'
    rep_id = db.Column(db.String(36), db.ForeignKey('representative.id'))
    representative = db.relationship('Representative',
        primaryjoin='RepresentativeState.rep_id == Representative.id',
        backref = db.backref('states'))
    state_date = db.Column(db.DateTime)
    last_update_date = db.Column(db.DateTime)
    web_site_url = db.Column(db.Text)
    original_name = db.Column(db.Text)
    activities = db.Column(db.Text)
    info_members = db.Column(db.Text)
    code_of_conduct = db.Column(db.Text)
    other_code_of_conduct = db.Column(db.Unicode)
    members = db.Column(db.Float, nullable=True)

    state_financial_data_id = db.Column(db.String(36), db.ForeignKey('state_financial_data.id'))
    state_financial_data = db.relationship('StateFinancialData',
        primaryjoin='RepresentativeState.state_financial_data_id == StateFinancialData.id',
        uselist=False)

    head_id = db.Column(db.Unicode(36), db.ForeignKey('person.id'))
    head = db.relationship('Person',
                                  primaryjoin='RepresentativeState.head_id==Person.id',
                                  uselist=False)

    legal_id = db.Column(db.Unicode(36), db.ForeignKey('person.id'))
    legal = db.relationship('Person',
                                  primaryjoin='RepresentativeState.legal_id==Person.id',
                                  uselist=False)
    goals = db.Column(db.Text)
    networking = db.Column(db.Text)
    acronym = db.Column(db.Text)
    legal_status = db.Column(db.Text)
    be_office_id = db.Column(db.BigInteger, db.ForeignKey('contact.id'), nullable=True)
    be_office = db.relationship('Contact',
                                  primaryjoin='RepresentativeState.be_office_id==Contact.id',
                                  uselist=False)

    head_office_id = db.Column(db.BigInteger, db.ForeignKey('contact.id'), nullable=True)
    head_office = db.relationship('Contact',
                                  primaryjoin='RepresentativeState.head_office_id==Contact.id',
                                  uselist=False)
    main_category_id = db.Column(db.BigInteger, db.ForeignKey('category.id'), nullable=False)
    main_category = db.relationship('Category',
                                  primaryjoin='RepresentativeState.main_category_id==Category.id',
                                  uselist=False)
    sub_category_id = db.Column(db.BigInteger, db.ForeignKey('category.id'), nullable=False)
    sub_category = db.relationship('Category',
                                  primaryjoin='RepresentativeState.sub_category_id==Category.id',
                                  uselist=False)
    members_25 = db.Column(db.BigInteger, nullable=True)
    members_50 = db.Column(db.BigInteger, nullable=True)
    members_75 = db.Column(db.BigInteger, nullable=True)
    members_100 = db.Column(db.BigInteger, nullable=True)
    members_fte = db.Column(db.Float, nullable=True)

    number_of_natural_persons = db.Column(db.BigInteger, nullable=True)

    registration_date = db.Column(db.DateTime)
    activity_consult_committees = db.Column(db.Unicode)
    activity_eu_legislative = db.Column(db.Unicode)
    activity_expert_groups = db.Column(db.Unicode)
    activity_high_level_groups = db.Column(db.Unicode)
    activity_industry_forums = db.Column(db.Unicode)
    activity_inter_groups = db.Column(db.Unicode)
    activity_other = db.Column(db.Unicode)
    activity_relevant_comm = db.Column(db.Unicode)

    def getAccreditations(self):
        qry = db.session.execute("select count(*) from representative_state where rep_id = :rid and state_date > :sd",{'rid':self.rep_id,'sd':self.state_date})
        later_states = qry.fetchone()[0]
        if self.representative.status != 'active' or later_states > 0:
            qry = db.session.execute("select e.name from acc_appearance left join Accreditation on Accreditation.id = acc_appearance.accreditation_id \
            left join Person AS p on Accreditation.person_id = p.id left join Entity AS e on p.entity_id = e.id \
            where Accreditation.representative_id = :rid and date <= :sdate and (disappeared_date is null or disappeared_date > :sdate)",{'rid':self.rep_id,'sdate':self.state_date})
            res = []
            for row in qry:
                res.append(row['name'])
        else:
            res = [a.as_name() for a in self.representative.accreditations if a.status == 'active']
        return res

    @classmethod
    def create(cls, data):
        cls = cls()
        return cls.update(data)

    def update(self, data):
        self.members = data.get('members')
        self.members_25 = data.get('members_25')
        self.members_50 = data.get('members_50')
        self.members_75 = data.get('members_75')
        self.members_100 = data.get('members_100')
        self.members_fte = data.get('members_fte')
        self.activity_consult_committees = data.get('activity_consult_committees')
        self.activity_eu_legislative = data.get('activity_eu_legislative')
        self.activity_expert_groups = data.get('activity_expert_groups')
        self.activity_high_level_groups = data.get('activity_high_level_groups')
        self.activity_industry_forums = data.get('activity_industry_forums')
        self.activity_inter_groups = data.get('activity_inter_groups')
        self.activity_other = data.get('activity_other')
        self.activity_relevant_comm = data.get('activity_relevant_comm')
        self.state_date = data.get('state_date')
        self.last_update_date = data.get('last_update_date')
        self.registration_date = data.get('registration_date')
        self.rep_id = data.get('rep_id')
        self.state_financial_data = data.get('state_financial_data')
        self.head = data.get('head')
        self.info_members = data.get('info_members')
        self.number_of_natural_persons = data.get('number_of_natural_persons')
        self.legal = data.get('legal')
        self.head_office = data.get('head_office')
        self.be_office = data.get('be_office')
        self.original_name = data.get('original_name')
        self.activities = data.get('activities')
        self.code_of_conduct = data.get('code_of_conduct')
        self.other_code_of_conduct = data.get('other_code_of_conduct')
        self.acronym = data.get('acronym')
        self.legal_status = data.get('legal_status')
        self.main_category = data.get('main_category')
        self.sub_category = data.get('sub_category')
        self.web_site_url = data.get('web_site_url')
        self.goals = data.get('goals')
        self.networking = data.get('networking')
        db.session.add(self)
        return self

    def as_shallow(self):
        d = {}
        d.update({
            'identification_code': self.representative.identification_code,
            'name': self.original_name,
            'acronym': self.acronym,
            'goals': self.goals,
            'structure_members': self.representative.structure_members,
            'activities': self.activities,
            'activity_eu_legislative' : self.activity_eu_legislative,
            'activity_other': self.activity_other,
            'activity_consult_committees' : self.activity_consult_committees,
            'activity_eu_legislative' : self.activity_eu_legislative,
            'activity_expert_groups' : self.activity_expert_groups,
            'activity_high_level_groups': self.activity_high_level_groups,
            'activity_industry_forums': self.activity_industry_forums,
            'activity_inter_groups': self.activity_inter_groups,
            'activity_relevant_comm': self.activity_relevant_comm,
            'head_office' : self.head_office.as_shallow(),
            'be_office' : self.be_office.as_shallow() if self.be_office else None,
            'networking': self.networking,
            'code_of_conduct': self.code_of_conduct,
            'web_site_url': self.web_site_url,
            'legal_status': self.legal_status,
            'members': self.members,
            'members_fte': self.members_fte,
            'financial_data': self.state_financial_data.as_shallow(turnovers=True),
            'custom sources': [cs.as_shallow() for cs in self.state_financial_data.custsources],
            'head_person': self.head.as_shallow(),
            'legal_person': self.legal.as_shallow(),
            'main_category': self.main_category.as_shallow() if self.main_category else None,
            'sub_category': self.sub_category.as_shallow() if self.sub_category else None,
            'representative_id': self.representative.id,
            'state_date' : self.state_date,
            'info_members': self.info_members,
            'actions': [act.get_action_name() for act in self.state_action_fields],
            'interests': [intrst.get_interest_name() for intrst in self.interests],
            'accreditations': self.getAccreditations(),
            'registration_date': self.registration_date,
            })
        return d

    def as_dict(self):
        d = self.as_shallow()
        d.update({
            'main_category': self.main_category.as_shallow() if self.main_category else None,
            'sub_category': self.sub_category.as_shallow() if self.sub_category else None,
            'head': self.head.as_shallow() if self.head else None,
            'legal': self.legal.as_shallow() if self.legal else None,
            'head_office': self.head_office.as_shallow() if self.head_office else None,
            })
        return d

    @classmethod
    def by_id(cls, id):
        return cls.by_attr(cls.rep_id,id)

    @classmethod
    def by_date_id(cls,tdate,tid):
        q = db.session.query(cls)
        q = q.filter(cls.state_date==tdate)
        q = q.filter(cls.rep_id==tid)
        return q.first()

    @classmethod
    def by_state_id(cls, id):
        return cls.by_attr(cls.id,id)

    @classmethod
    def by_id_last(cls,id):
        q = db.session.query(cls)
        q = q.order_by('state_date DESC')
        q = q.filter(RepresentativeState.rep_id==id)
        # obj = q.first()
        i = 0
        tlist = []
        meetings = []
        tactive = 0
        taccs = []
        for state in q:
            if i == 0:
                tactive = 0
                if(state.representative.status == 'active'):
                    tactive = 1
                    # state.accreditations = [a.as_name() for a in state.representative.accreditations if a.status == 'active']
                taccs = [a.as_name() for a in state.representative.accreditations if a.status == 'active']

                tlist = [state]
                meetings = [m.as_dict() for m in state.representative.meetings_sorted()]
                i+=1
            else:
                tlist.append({'id':state.id,'state_date':state.state_date,'financial_data':state.state_financial_data.as_mini()})
        return {'results':tlist,'active':tactive,'meetings':meetings,'accs':taccs}

    @classmethod
    def by_id_last_dedupe(cls,id):
        qry = db.session.execute("select alter_ego from alter_ego where representative_id = :rid",{'rid':id})
        res = [id]
        for alter_ego in qry:
            res.append(alter_ego['alter_ego'])
        q = db.session.query(cls)
        q = q.order_by('state_date DESC')
        q = q.filter(RepresentativeState.rep_id.in_(res))
        # obj = q.first()
        i = 0
        tlist = []
        meetings = []
        tactive = 0
        taccs = []
        for state in q:
            if i == 0:
                tactive = 0
                if(state.representative.status == 'active'):
                    tactive = 1
                    # state.accreditations = [a.as_name() for a in state.representative.accreditations if a.status == 'active']
                taccs = [a.as_name() for a in state.representative.accreditations if a.status == 'active']

                tlist = [state]
                meetings = [m.as_dict() for m in state.representative.meetings_sorted()]
                i+=1
            else:
                tlist.append({'id':state.id,'state_date':state.state_date,'financial_data':state.state_financial_data.as_mini()})
        return {'results':tlist,'active':tactive,'meetings':meetings,'accs':taccs}

    @classmethod
    def by_attr(cls, attr, value):
        q = db.session.query(cls)
        q = q.order_by('state_date DESC')
        q = q.filter(attr==value)
        return q.first()

    @classmethod
    def all(cls):
        q = db.session.query(cls)
        q = q.order_by('state_date DESC')
        return q

    def __repr__(self):
        return "<RepresentativeState(%s)>" % (self.rep_id)

class rep_appearance(db.Model):
    __tablename__ = 'rep_appearance'

    id = db.Column(db.BigInteger,primary_key=True)
    representative_id = db.Column(db.String(36))
    identification_code = db.Column(db.String(30))
    date = db.Column(db.DateTime)
    disappeared_date = db.Column(db.DateTime)
    disappeared = db.Column(db.Boolean)

    @classmethod
    def create(cls, data):
        cls = cls()
        return cls.update(data)

    def update(self, data):
        self.identification_code = data.get('identification_code')
        self.date = data.get('date')
        self.disappeared = data.get('disappeared')
        self.representative_id = data.get('representative_id')
        db.session.add(self)
        return self

class acc_appearance(db.Model):
    __tablename__ = 'acc_appearance'

    id = db.Column(db.BigInteger,primary_key=True)
    accreditation_id = db.Column(db.String(36))
    date = db.Column(db.DateTime)
    disappeared_date = db.Column(db.DateTime)

    @classmethod
    def create(cls, data):
        cls = cls()
        return cls.update(data)

    def update(self, data):
        self.accreditation_id = data.get('accreditation_id')
        self.date = data.get('date')
        self.disappeared = data.get('disappeared')
        db.session.add(self)
        return self

class StateFinancialData(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'state_financial_data'

    representative_id = db.Column(db.String(36), db.ForeignKey('representative.id'))

    turnover_min = db.Column(db.BigInteger, nullable=True)
    turnover_max = db.Column(db.BigInteger, nullable=True)
    turnover_absolute = db.Column(db.BigInteger, nullable=True)
    cost_min = db.Column(db.BigInteger, nullable=True)
    cost_max = db.Column(db.BigInteger, nullable=True)
    cost_absolute = db.Column(db.BigInteger, nullable=True)
    direct_rep_costs_min = db.Column(db.BigInteger, nullable=True)
    direct_rep_costs_max = db.Column(db.BigInteger, nullable=True)
    total_budget = db.Column(db.BigInteger, nullable=True)
    public_financing_total = db.Column(db.BigInteger, nullable=True)
    public_financing_infranational = db.Column(db.BigInteger, nullable=True)
    public_financing_national = db.Column(db.BigInteger, nullable=True)
    eur_sources_grants = db.Column(db.BigInteger, nullable=True)
    eur_sources_procurement = db.Column(db.BigInteger, nullable=True)
    other_sources_donation = db.Column(db.BigInteger, nullable=True)
    other_sources_contributions = db.Column(db.BigInteger, nullable=True)
    other_sources_total = db.Column(db.BigInteger, nullable=True)
    no_clients = db.Column(db.Unicode, nullable=True)

    eur_sources_procurement_src = db.Column(db.Unicode)
    eur_sources_grants_src = db.Column(db.Unicode)
    other_financial_information = db.Column(db.Unicode)
    new_organisation = db.Column(db.Unicode)

    status = db.Column(db.Unicode)
    start_date = db.Column(db.DateTime)
    end_date = db.Column(db.DateTime)
    type = db.Column(db.Unicode)

    def update_values(self, data):
        self.representative = data.get('representative')
        self.turnover_min = to_integer(data.get('turnover_min'))
        self.turnover_max = to_integer(data.get('turnover_max'))
        self.turnover_absolute = to_integer(data.get('turnover_absolute'))
        self.cost_min = to_integer(data.get('cost_min'))
        self.cost_max = to_integer(data.get('cost_max'))
        self.cost_absolute = to_integer(data.get('cost_absolute'))
        self.direct_rep_costs_min = to_integer(data.get('direct_rep_costs_min'))
        self.direct_rep_costs_max = to_integer(data.get('direct_rep_costs_max'))
        self.total_budget = to_integer(data.get('total_budget'))
        self.public_financing_total = to_integer(data.get('public_financing_total'))
        self.public_financing_infranational = to_integer(data.get('public_financing_infranational'))
        self.public_financing_national = to_integer(data.get('public_financing_national'))
        self.eur_sources_grants = to_integer(data.get('eur_sources_grants'))
        self.eur_sources_procurement = to_integer(data.get('eur_sources_procurement'))
        self.eur_sources_procurement_src = data.get('eur_sources_procurement_src')
        self.eur_sources_grants_src = data.get('eur_sources_grants_src')
        self.other_financial_information = data.get('other_financial_information')

        self.other_sources_donation = to_integer(data.get('other_sources_donation'))
        self.other_sources_contributions = to_integer(data.get('other_sources_donation'))
        self.other_sources_total = to_integer(data.get('other_sources_total'))
        self.status = data.get('status')
        self.no_clients = data.get('no_clients')

        self.start_date = data.get('start_date')
        self.end_date = data.get('end_date')
        self.type = data.get('type')

    def cascade_delete(self):
        for t in self.turnovers:
            t.delete()

    @classmethod
    def by_rsd(cls, representative, start_date):
        q = db.session.query(cls)
        q = q.filter(cls.representative_id==representative.id)
        q = q.filter(cls.start_date==start_date)
        return q.first()

    @classmethod
    def by_all(cls, fd_arr):
        q = db.session.query(cls)
        q = q.filter(cls.representative_id==fd_arr.get('representative',None).id)
        #q = q.filter(cls.start_date==fd_arr.get('start_date',None))
        q = q.filter(cls.start_date==fd_arr.get('start_date',None))
        q = q.filter(cls.end_date ==fd_arr.get('end_date',None))
        q = q.filter(cls.turnover_min ==to_integer(fd_arr.get('turnover_min',None)))
        q = q.filter(cls.turnover_max ==to_integer(fd_arr.get('turnover_max',None)))
        q = q.filter(cls.turnover_absolute ==to_integer(fd_arr.get('turnover_absolute',None)))
        q = q.filter(cls.cost_min ==to_integer(fd_arr.get('cost_min',None)))
        q = q.filter(cls.cost_max ==to_integer(fd_arr.get('cost_max',None)))
        q = q.filter(cls.cost_absolute ==to_integer(fd_arr.get('cost_absolute',None)))
        q = q.filter(cls.direct_rep_costs_min ==to_integer(fd_arr.get('direct_rep_costs_min',None)))
        q = q.filter(cls.direct_rep_costs_max ==to_integer(fd_arr.get('direct_rep_costs_max',None)))
        q = q.filter(cls.total_budget ==to_integer(fd_arr.get('total_budget',None)))
        q = q.filter(cls.public_financing_total ==to_integer(fd_arr.get('public_financing_total',None)))
        q = q.filter(cls.public_financing_national ==to_integer(fd_arr.get('public_financing_national',None)))
        q = q.filter(cls.public_financing_infranational ==to_integer(fd_arr.get('public_financing_infranational',None)))
        q = q.filter(cls.eur_sources_grants ==to_integer(fd_arr.get('eur_sources_grants',None)))
        q = q.filter(cls.eur_sources_procurement ==to_integer(fd_arr.get('eur_sources_procurement',None)))
        q = q.filter(cls.other_sources_donation ==to_integer(fd_arr.get('other_sources_donation',None)))
        q = q.filter(cls.other_sources_contributions ==to_integer(fd_arr.get('other_sources_contributions',None)))
        q = q.filter(cls.other_sources_total == to_integer(fd_arr.get('other_sources_total',None)))
        return q.first()

    def as_mini(self):
        d = {}
        d.update({
            'start_date': self.start_date,
            'end_date': self.end_date,
            'turnover_min': self.turnover_min,
            'turnover_max': self.turnover_max,
            'turnover_absolute': self.turnover_absolute,
            'cost_min': self.cost_min,
            'cost_max': self.cost_max,
            'cost_absolute': self.cost_absolute,
            'direct_rep_costs_min': self.direct_rep_costs_min,
            'direct_rep_costs_max': self.direct_rep_costs_max,
            'total_budget': self.total_budget,
            'public_financing_total': self.public_financing_total,
            'public_financing_national': self.public_financing_national,
            'public_financing_infranational': self.public_financing_infranational,
            'eur_sources_grants': self.eur_sources_grants,
            'eur_sources_procurement': self.eur_sources_procurement,
            'other_sources_donation': self.other_sources_donation,
            'other_sources_contributions': self.other_sources_contributions,
            'other_sources_total': self.other_sources_total,
            })
        return d

    def as_shallow(self, turnovers=False):
        d = super(StateFinancialData, self).as_dict()
        d.update({
            'start_date': self.start_date,
            'end_date': self.end_date,
            'turnover_min': self.turnover_min,
            'turnover_max': self.turnover_max,
            'turnover_absolute': self.turnover_absolute,
            'cost_min': self.cost_min,
            'cost_max': self.cost_max,
            'cost_absolute': self.cost_absolute,
            'direct_rep_costs_min': self.direct_rep_costs_min,
            'direct_rep_costs_max': self.direct_rep_costs_max,
            'total_budget': self.total_budget,
            'public_financing_total': self.public_financing_total,
            'public_financing_national': self.public_financing_national,
            'public_financing_infranational': self.public_financing_infranational,
            'eur_sources_grants': self.eur_sources_grants,
            'eur_sources_procurement': self.eur_sources_procurement,
            'other_sources_donation': self.other_sources_donation,
            'other_sources_contributions': self.other_sources_contributions,
            'other_sources_total': self.other_sources_total,
            'other_financial_information': self.other_financial_information,
            })
        if self.id:
            d['id']=self.id
        if turnovers:
            d['turnovers'] = [t.as_dict(financial_data=False) for t in self.turnovers]
        return d

    def as_dict(self):
        d = self.as_shallow(turnovers=True)
        d.update({
#            'representative': self.representative.as_shallow() if self.representative else None,
#            'representative_state': self.representative_state.as_shallow() if self.representative_state else None,
            })
        return d

    def __repr__(self):
        return "<StateFinancialData(%s,%r)>" % (self.start_date, self.representative_state)


# RepresentativeState.state_financial_datas = db.relationship(StateFinancialData,
#             lazy='dynamic',
#             backref=db.backref('representative_state',
#                 uselist=False,
#                 ))
Representative.state_financial_datas = db.relationship(StateFinancialData,
            lazy='dynamic',
            backref=db.backref('representative',
                uselist=False,
                ))


class StateFinancialTurnover(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'state_financial_turnover'

    state_financial_data_id = db.Column(db.String(36), db.ForeignKey('state_financial_data.id'))
    entity_id = db.Column(db.String(36), db.ForeignKey('entity.id'))

    min = db.Column(db.Integer)
    max = db.Column(db.Integer)
    status = db.Column(db.Unicode)

    def update_values(self, data):
        self.state_financial_data = data.get('state_financial_data')
        self.entity = data.get('entity')
        self.status = data.get('status')

        self.min = data.get('min')
        self.max = data.get('max')

    @classmethod
    def by_fde(cls, state_financial_data, entity):
        q = db.session.query(cls)
        q = q.filter(cls.state_financial_data_id==state_financial_data.id)
        q = q.filter(cls.entity_id==entity.id)
        return q.first()

    def as_shallow(self):
        d = super(StateFinancialTurnover, self).as_dict()
        d.update({
            'state_financial_data_id': self.state_financial_data_id,
            'entity_id': self.entity_id,
            'name': self.entity.name if self.entity else None, # curve
            'min': self.min,
            'max': self.max,
            'status': self.status,
            })
        return d

    def as_dict(self, financial_data=True, entity=True):
        #d = super(StateFinancialTurnover, self).as_dict()
        d = self.as_shallow()
        d.update({
            'uri': self.uri,
            })
        if financial_data:
            d['financial_data'] = self.state_financial_data.as_shallow()
        if entity:
            d['entity'] = self.entity.as_shallow()
        return d

    def __repr__(self):
        return "<StateFinancialTurnover(%r,%r)>" % (self.state_financial_data, self.entity)


StateFinancialData.turnovers = db.relationship('StateFinancialTurnover',
            lazy='dynamic',
            backref=db.backref('state_financial_data',
                uselist=False,
                ))

Entity.turnovers = db.relationship('StateFinancialTurnover',
            lazy='dynamic',
            backref=db.backref('entity',
                uselist=False,
                ))

class StateAssociatedAction(db.Model, ApiEntityMixIn):
    __tablename__ = 'state_associated_action'

    id = db.Column(db.BigInteger, primary_key=True)
    state_id = db.Column(db.String(36), db.ForeignKey('representative_state.id'))
    action_id = db.Column(db.BigInteger(), db.ForeignKey('action_field.id'))

    action = db.relationship(ActionField,
            backref=db.backref('associated_rep_states',
                lazy='dynamic',
            ))

    state = db.relationship(RepresentativeState,
    	backref=db.backref('state_action_fields', lazy='select'))

    @classmethod
    def create(cls, data):
        cls = cls()
        return cls.update(data)

    def update(self, data):
        self.state = data.get('state')
        self.action = data.get('action')

    def get_action_name(self):
        return self.action.action

    def as_shallow(self):
#        d = super(StateAssociatedAction, self).as_dict()
        d = {}
        d.update({
#             'state_id': self.state_id,
#             'action_field_id': self.action_id,
            'action': self.action.action
            })
        return d

    @classmethod
    def by_rpa(cls, state, action):
#    	print action.id
#    	print state.id
        q = db.session.query(cls)
        q = q.filter(cls.action_id==action.id)
        q = q.filter(cls.state_id==state.id)
        return q.first()

    def __repr__(self):
        return "<StateAssociatedAction(%s,%r)>" % (self.id, self.action)

class StateAssociatedInterest(db.Model):

    __tablename__ = 'state_associated_interest'

    id = db.Column(db.BigInteger, primary_key=True)
    state_id = db.Column(db.String(36), db.ForeignKey('representative_state.id'))
    interest_id = db.Column(db.BigInteger(), db.ForeignKey('interest.id'))

    index = db.Index('idx_rpi', 'representative_id', 'interest_id')

    interest = db.relationship(Interest,
            backref=db.backref('associated_rep_states',
                lazy='dynamic',
            ))

    state = db.relationship(RepresentativeState,
        uselist=False,
        backref=db.backref('interests',
            lazy='select',
            ))

    @classmethod
    def create(cls, data):
        cls = cls()
        return cls.update(data)

    def update(self, data):
        self.state = data.get('state')
        self.interest = data.get('interest')

    def get_interest_name(self):
        return self.interest.interest

    def as_shallow(self):
#        d = super(StateAssociatedInterest, self).as_dict()
        d = {}
        d.update({
#             'state_id': self.state_id,
#             'interest_id': self.interest_id,
                'interest': self.interest.interest
            })
        return d

    @classmethod
    def by_rpi(cls, state, interest):
        q = db.session.query(cls)
        q = q.filter(cls.interest_id==interest.id)
        q = q.filter(cls.state_id==state.id)
        return q.first()

    def __repr__(self):
        return "<StateAssociatedInterest(%s,%r)>" % (self.id, self.interest)

class StateCustomIncome(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'state_custom_income'

    state_financial_data_id = db.Column(db.String(36), db.ForeignKey('state_financial_data.id'))
    name = db.Column(db.Unicode)
    amount = db.Column(db.BigInteger)
    type = db.Column(db.Unicode)
    status = db.Column(db.Unicode)

    def update_values(self, data):
        self.state_financial_data = data.get('state_financial_data')
        self.status = data.get('status')

        self.name = data.get('name')
        self.amount = data.get('amount')
        self.type = data.get('type')

    @classmethod
    def by_fdn(cls, state_financial_data, name):
        q = db.session.query(cls)
        q = q.filter(cls.state_financial_data_id==state_financial_data.id)
        q = q.filter(cls.name==name)
        return q.first()

    @classmethod
    def by_all(cls, ci_array):
        q = db.session.query(cls)
        q = q.filter(cls.state_financial_data_id == ci_array['state_financial_data'].id)
        q = q.filter(cls.name == ci_array['name'])
        q = q.filter(cls.amount == ci_array['amount'])
        q = q.filter(cls.type == ci_array['type'])

    def as_shallow(self):
        d = super(StateCustomIncome, self).as_dict()
        d.update({
            'financial_data_id': self.state_financial_data_id,
            'type': self.type,
            'amount': self.amount,
            'name': self.name,
            'status': self.status,
            })
        return d

    def as_dict(self, financial_data=True, entity=True):
        d = super(StateCustomIncome, self).as_dict()
        d.update({
            'uri': self.uri,
            'name': self.name,
            'amount': self.amount,
            'type': self.type,
            'status': self.status,
            })
        if financial_data:
            d['state_financial_data'] = self.state_financial_data.as_shallow()
        return d

    def __repr__(self):
        return "<StateCustomeIncome(%r,%s:%s)>" % (self.state_financial_data, self.name, self.amount)

StateFinancialData.custsources = db.relationship('StateCustomIncome',
            lazy='dynamic',
            backref=db.backref('state_financial_data',
                uselist=False,
                ))

class OrganisationState(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'organisation_state'

    entity_id = db.Column(db.String(36), db.ForeignKey('entity.id'))
    number_of_members = db.Column(db.BigInteger, nullable=True)
    status = db.Column(db.Unicode)

    def update_values(self, data):
        self.entity = data.get('entity')
        self.number_of_members = data.get('number_of_members')
        self.status = data.get('status')

    @classmethod
    def by_name(cls, name):
        q = db.session.query(cls)
        q = q.join(Entity)
        q = q.filter(Entity.name==name)
        return q.first()

    @classmethod
    def by_all(cls, data):
        q = db.session.query(cls)
        q = q.join(Entity)
        q = q.filter(Entity.name==data['name'])
        q = q.filter(cls.number_of_members == data['number_of_members'])
        return q.first()

    def as_shallow(self):
        d = super(OrganisationState, self).as_dict()
        d.update({
            'id': self.id,
            'entity': self.entity_id,
            'uri': self.uri,
            'name': self.entity.name, # drupal
            'acronym': self.entity.acronym, # drupal
            'status': self.status,
            'number_of_members': self.number_of_members
            })
        return d

    def as_dict(self):
        d = self.as_shallow()
        d.update({
            'entity': self.entity.as_shallow(),
            'state_memberships': [m.as_dict(organisation_state=False) for m in self.state_memberships]
            })
        return d

    def __repr__(self):
        return "<OrganisationState(%s,%r)>" % (self.id, self.entity)


Entity.organisation_state = db.relationship(OrganisationState,
        uselist=False,
        backref=db.backref('entity'))


class OrganisationStateMembership(db.Model, RevisionedMixIn, ApiEntityMixIn):
    __tablename__ = 'organisation_state_membership'

    organisation_state_id = db.Column(db.String(36), db.ForeignKey('organisation_state.id'))
    representative_state_id = db.Column(db.String(36), db.ForeignKey('representative_state.id'))
    status = db.Column(db.Unicode)

    def update_values(self, data):
        self.organisation_state = data.get('organisation_state')
        self.representative_state = data.get('representative_state')
        self.status = data.get('status')

    @classmethod
    def by_rpo(cls, representative_state, organisation_state):
        q = db.session.query(cls)
        q = q.filter(cls.representative_state_id==representative_state.id)
        q = q.filter(cls.organisation_state_id==organisation_state.id)
        return q.first()

    def as_shallow(self):
        d = super(OrganisationStateMembership, self).as_dict()
        d.update({
            'organisation_state_id': self.organisation_state_id,
            'representative_state_id': self.representative_state_id,
            })
        return d

    def as_dict(self, organisation_state=True, representative_state=True):
        d = super(OrganisationStateMembership, self).as_dict()
        d.update({
            'uri': self.uri,
            'status': self.status,
            })
        if organisation_state:
            d['organisation_state'] = self.organisation_state.as_shallow()
        if representative_state:
            d['representative_state'] = self.representative_state.as_shallow()
        return d


    def __repr__(self):
        return "<OrganisationStateMembership(%s,%r,%r)>" % (self.id, self.representative_state, self.organisation_state)

OrganisationStateMembership.organisation_state = db.relationship(OrganisationState,
        uselist=False,
        backref=db.backref('state_memberships',
            lazy='dynamic',
            ))

OrganisationStateMembership.representative_state = db.relationship(RepresentativeState,
        uselist=False,
        backref=db.backref('organisation_state_memberships',
            lazy='dynamic',
            ))

class LastDate(db.Model):
    __tablename__ = 'lastdate'
    lastdate = db.Column(db.DateTime)
    id = db.Column(db.BigInteger, primary_key=True)
    @classmethod
    def create(cls, data):
        obj = cls()
        obj.update(data)
        return obj

    def update_values(self, data):
        self.lastdate = data.get('lastdate')

    @classmethod
    def get_lastdate(cls):
        q = db.session.query(cls)
        return q.first()

    def update(self, data):
        self.update_values(data)
        if not self in db.session:
            db.session.add(self)
        db.session.flush()
        return self
